#include "stdafx.h"
#include "resource.h"

#include "WtlFixed/atlmisc.h"
#include <atlframe.h>
#include <atlctrls.h>
#include <atlctrlw.h>
#include "WtlFixed/atlsplit.h"
#include <atldlgs.h>
#include <atlctrlx.h>
#include <atlcrack.h>
#include <atlddx.h>
#include <wininet.h>
#include <MsHTML.h>

#include "ChevronHandler.h"
#include "Mtl.h"

namespace MTL
{

// Helper for creating default FORMATETC from cfFormat
LPFORMATETC	_MtlFillFormatEtc(
	LPFORMATETC	lpFormatEtc, CLIPFORMAT	cfFormat, LPFORMATETC lpFormatEtcFill)
{
	ATLASSERT(lpFormatEtcFill != NULL);
	if (lpFormatEtc	== NULL	&& cfFormat	!= 0)
	{
		lpFormatEtc	= lpFormatEtcFill;
		lpFormatEtc->cfFormat =	cfFormat;
		lpFormatEtc->ptd = NULL;
		lpFormatEtc->dwAspect =	DVASPECT_CONTENT;
		lpFormatEtc->lindex	= -1;
		lpFormatEtc->tymed = (DWORD) -1;
	}
	return lpFormatEtc;
}

bool MtlIsDataAvailable(IDataObject* pDataObject, CLIPFORMAT cfFormat, LPFORMATETC lpFormatEtc)
{
	ATLASSERT(pDataObject != NULL);

	// fill	in FORMATETC struct
	FORMATETC formatEtc;
	lpFormatEtc	= MTL::_MtlFillFormatEtc(lpFormatEtc, cfFormat,	&formatEtc);

	// attempt to get the data
	return pDataObject->QueryGetData(lpFormatEtc) == S_OK;
}

bool MtlGetDropFileName(IDataObject* pDataObject, CSimpleArray<CString>& arrFileNames)
{
	if (!MTL::MtlIsDataAvailable(pDataObject, CF_HDROP))
		return false;

	FORMATETC formatetc	= {	CF_HDROP, NULL,	DVASPECT_CONTENT, -1, TYMED_HGLOBAL	};
	STGMEDIUM stgmedium;
	HRESULT	hr = pDataObject->GetData(&formatetc, &stgmedium);
	if (FAILED(hr) || stgmedium.hGlobal	== NULL) {
		return false;
	}

	HDROP hDropInfo	= (HDROP)stgmedium.hGlobal;

	UINT nFiles	= ::DragQueryFile(hDropInfo, (UINT)-1, NULL, 0);
	for	(UINT iFile	= 0; iFile < nFiles; iFile++) {
		TCHAR szFileName[_MAX_PATH];
		::DragQueryFile(hDropInfo, iFile, szFileName, _MAX_PATH);
		arrFileNames.Add(CString(szFileName));
	}
	::DragFinish(hDropInfo);// required?

	::ReleaseStgMedium(&stgmedium);

	if (arrFileNames.GetSize() > 0)
		return true;
	else
		return false;
}

// Implementation
bool MtlGetHGlobalText(IDataObject*	pDataObject, CString& strText, CLIPFORMAT cfFormat)
{
	bool bResult = false;

	if (!MTL::MtlIsDataAvailable(pDataObject, cfFormat))
		return false;

	FORMATETC formatetc	= {	cfFormat, NULL,	DVASPECT_CONTENT, -1, TYMED_HGLOBAL	};
	STGMEDIUM stgmedium;
	HRESULT	hr = pDataObject->GetData(&formatetc, &stgmedium);
	if (SUCCEEDED(hr)) {
		if (stgmedium.hGlobal != NULL) {
			HGLOBAL	hText =	stgmedium.hGlobal;
			strText	= reinterpret_cast<LPSTR>(::GlobalLock(hText));
			::GlobalUnlock(hText);
			bResult	= true;
		}
		::ReleaseStgMedium(&stgmedium);
	}

	return bResult;
}

HDROP MtlCreateDropFile(CSimpleArray<CString>& arrFiles)
{
	if (arrFiles.GetSize() == 0)
		return NULL;

	//filename\0...\0filename\0...\0filename\0\0
	int	nLen = 0;

	int	i;
	for	(i = 0;	i <	arrFiles.GetSize();	++i) {
		nLen +=	arrFiles[i].GetLength();
		nLen +=	1;	// for '\0'	separator
	}
	nLen +=	1;		// for the last	'\0'

	HDROP hDrop	= (HDROP)::GlobalAlloc(GHND, sizeof(DROPFILES) + nLen*sizeof(TCHAR));
	if (hDrop == NULL)
		return NULL;

	LPDROPFILES	lpDropFiles;
	lpDropFiles	= (LPDROPFILES)::GlobalLock(hDrop);
	lpDropFiles->pFiles	= sizeof(DROPFILES);
	lpDropFiles->pt.x =	0;
	lpDropFiles->pt.y =	0;
	lpDropFiles->fNC = FALSE;

#ifdef _UNICODE
	lpDropFiles->fWide = TRUE;
#else
	lpDropFiles->fWide = FALSE;
#endif

	TCHAR* psz = (TCHAR*)(lpDropFiles +	1);
	for	(i = 0;	i <	arrFiles.GetSize();	++i) {
		::lstrcpy(psz, arrFiles[i]);
		psz	+= arrFiles[i].GetLength() + 1;// skip a '\0' separator
	}

	::GlobalUnlock(hDrop);
	return hDrop;
}

}