#include "stdafx.h"
#include "resource.h"

#include "WtlFixed/atlmisc.h"
#include <atlframe.h>
#include <atlctrls.h>
#include <atlctrlw.h>
#include "WtlFixed/atlsplit.h"
#include <atldlgs.h>
#include <atlctrlx.h>
#include <atlcrack.h>
#include <atlddx.h>
#include <wininet.h>
#include <MsHTML.h>

#include "ChevronHandler.h"
#include "Mtl.h"

namespace MTL
{

bool MtlCreateInternetShortcutFile(const CString& strFileName, const CString& strUrl)
{//	Thanks to <http://www.nt.sakura.ne.jp/~miwaki/>
	CComPtr<IUniformResourceLocator> spUrl;
	HRESULT	hr = ::CoCreateInstance(CLSID_InternetShortcut,	NULL,
				CLSCTX_ALL,	IID_IUniformResourceLocator, (void**)&spUrl);
	if (FAILED(hr))
		return false;

	CComPtr<IPersistFile> spPf;
	hr = spUrl->QueryInterface(IID_IPersistFile, (void**)&spPf);
	if (FAILED(hr))
		return false;

	hr = spUrl->SetURL(strUrl, 0);
	if (FAILED(hr))
		return false;

	USES_CONVERSION;
	hr = spPf->Save(T2COLE((LPCTSTR)strFileName), TRUE);
	if (FAILED(hr))
		return false;

	return true;
}

bool MtlPreOpenFile(CString& strPath, const	CString& strArg)
{
	ATLTRACE(_T("MtlPreOpenFile(%s)\n"), strPath.Left(100));

	if (MtlIsExt(strPath, _T(".exe"))) {
		::ShellExecute(NULL, _T("open"), strPath, strArg, NULL,	SW_SHOWNORMAL);
		return true;// handled
	}

	if (!MtlIsExt(strPath, _T(".lnk")))
		return false;//	not	handled

	CComPtr<IShellLink>	spShellLink;
	// Create IShellLink Objec
	HRESULT	hr = ::CoCreateInstance(CLSID_ShellLink, NULL, CLSCTX_ALL, IID_IShellLink, (void **)&spShellLink);
	if (FAILED(hr))
		return false;

	CComQIPtr<IPersistFile>	spPersistFile(spShellLink);
	if (spPersistFile) {
		// load	shortcut
		USES_CONVERSION;
		hr = spPersistFile->Load(T2COLE(strPath), STGM_READ);
		if (SUCCEEDED(hr)) {
			// get link
			TCHAR			szFilePath[MAX_PATH];	szFilePath[0] =	0;
			WIN32_FIND_DATA	wfd;
			hr = spShellLink->GetPath(szFilePath, MAX_PATH,	&wfd, SLGP_UNCPRIORITY);
			if (FAILED(hr))
				return false;

			if (!MtlIsExt(szFilePath, _T(".exe"))) {
				strPath	= szFilePath;
				return false;
			}

			TCHAR szCmdLine[2048]; szCmdLine[0]	= _T('\0');
			hr = spShellLink->GetArguments(szCmdLine, 2048);

			if (!strArg.IsEmpty())
				::lstrcpyn(szCmdLine, strArg, 2048);

			int	nShowCmd = SW_SHOWNORMAL;
			hr = spShellLink->GetShowCmd(&nShowCmd);

			TCHAR szDirPath[MAX_PATH]; szDirPath[0]	= _T('\0');
			hr = spShellLink->GetWorkingDirectory(szDirPath, 2048);

			::ShellExecute(NULL, _T("open"), szFilePath, szCmdLine,	szDirPath, nShowCmd);
			return true;// handled
		}
	}

	return false;
}

CString	MtlMakeFavoriteToolTipText(const CString& strName_,	const CString& strUrl_,	int	cchTextMax)
{
	CString	strName	= MtlCompactString(strName_, cchTextMax);

	if (strUrl_.IsEmpty())
		return strName;

	int	nCount = cchTextMax	- strName.GetLength() -	1;// consider _('\n') too
	if (nCount <= 0)
		return strName;

	CString	strUrl;
	bool bRet =	MtlCompactPath(strUrl, strUrl_,	nCount);

	if (!bRet || strName ==	strUrl)
		return strName;
	else
		return strName + _T('\n') +	strUrl;
}

void MtlGetFavoritesOrder(CFavoritesOrder& order, const	CString& strDirPath)
{
	ATLASSERT(!strDirPath.IsEmpty());
	//+++ const	s_unknownOffset	= 12;
	const int s_unknownOffset =	12;

	CString	strRoot;
	if (!MtlGetFavoritesFolder(strRoot))
		return;
	MtlMakeSureTrailingBackSlash(strRoot);

	CString	strKeyName = _T("Software\\Microsoft\\Windows\\CurrentVersion\\Explorer\\MenuOrder\\Favorites");
	CString	strRelative	= strDirPath.Right(strDirPath.GetLength() -	strRoot.GetLength());
	if (!strRelative.IsEmpty())
		strKeyName += _T('\\') + strRelative;

	MtlRemoveTrailingBackSlash(strKeyName);
//	ATLTRACE(_T(" strKeyName(%s)\n"), strKeyName);

	CRegKey	rkOrder;
	LONG lRet =	rkOrder.Open(HKEY_CURRENT_USER,	strKeyName);
	if (lRet !=	ERROR_SUCCESS)
		return;

	DWORD dwSize = 0;
	DWORD dwType = REG_BINARY;
	lRet = ::RegQueryValueEx(rkOrder, _T("Order"), NULL, &dwType, NULL,	&dwSize);
	if (lRet !=	ERROR_SUCCESS || dwSize	< s_unknownOffset)
		return;

	BYTE* pByte	= (BYTE*)_alloca((dwSize+10) * sizeof(BYTE));
	::memset(pByte,	0, dwSize+10);

	lRet = ::RegQueryValueEx(rkOrder, _T("Order"), NULL, &dwType, pByte, &dwSize);
	if (lRet !=	ERROR_SUCCESS)
		return;

	BYTE* pBegin = pByte + ((_CFavoritesOrderData*)pByte)->size	+ s_unknownOffset;
	BYTE* pEnd = pByte + dwSize;

	while (pBegin <	pEnd) {
		_CFavoritesOrderData* pData	= (_CFavoritesOrderData*)pBegin;
		CItemIDList	idl(&pData->idl);
		CString	strName	= MtlGetFileName(idl.GetPath());
		if (!strName.IsEmpty())	{
//			ATLTRACE(_T("order(%s, %d)\n"),	strName, pData->priority);
			order.Add(strName, pData->priority);
		}

		pBegin += pData->size;
	}
}


bool MtlSetInternetZoneActionPolicy(DWORD dwAction,	DWORD dwPolicy)
{
	// create instance IInternetZoneManager
	CComPtr<IInternetZoneManager> spZoneMgr;
	HRESULT	hr = CoCreateInstance(CLSID_InternetZoneManager, NULL, CLSCTX_INPROC_SERVER,
		IID_IInternetZoneManager, (void**)&spZoneMgr);

	if (FAILED(hr))
		return false;

	// change security policy
	hr = spZoneMgr->SetZoneActionPolicy(URLZONE_INTERNET,
		dwAction, (BYTE*)&dwPolicy,	sizeof(dwPolicy), URLZONEREG_DEFAULT);

	if (FAILED(hr))
		return false;

	// set security	level custom
	// now the method is applied
	CVersional<ZONEATTRIBUTES> ZoneAttrib;
	hr = spZoneMgr->GetZoneAttributes(URLZONE_INTERNET,	&ZoneAttrib);

	if (FAILED(hr))
		return false;

	ZoneAttrib.dwTemplateCurrentLevel =	URLTEMPLATE_CUSTOM;
	hr = spZoneMgr->SetZoneAttributes(URLZONE_INTERNET,	&ZoneAttrib);
	if (FAILED(hr))
		return false;

	return true;
}

bool __MtlSkipChar(const CString& str, int&	nIndex,	TCHAR ch)
{
	ATLASSERT(nIndex < str.GetLength());

	if (str[nIndex]	== ch)
		++nIndex;

	if (nIndex >= str.GetLength())
		return false;
	else
		return true;
}

void _MtlCreateHrefUrlArray(CSimpleArray<CString>& arrUrl, const CString& strHtmlText)
{
	int	nLast =	-1;
	int	nFirst = 0;
	int	nLen = strHtmlText.GetLength();

	while (nFirst != -1	&& nFirst <	nLen)
	{
		nFirst = __MtlFind(strHtmlText,	nFirst,	_T("href"));
		nFirst += 4;
		if (nFirst >= nLen)
			break;

		if (!MTL::__MtlSkipChar(strHtmlText, nFirst, _T(' ')))
			break;


		if (strHtmlText[nFirst]	== _T('='))	// skip	a '='
			++nFirst;
		else
			break;							// illegal

		if (nFirst >= nLen)
			break;


		if (!MTL::__MtlSkipChar(strHtmlText, nFirst, _T(' ')))
			break;

		if (strHtmlText[nFirst]	== _T('\"')) {	// if "
			++nFirst;
			nLast =	__MtlFindChar(strHtmlText, nFirst, _T('\"'));
			if (nLast == -1)
				break;							// illegal

			CString	str	= strHtmlText.Mid(nFirst, nLast	- nFirst);
			arrUrl.Add(str);
		}
		else {
			nLast =	__MtlFindChar(strHtmlText, nFirst, _T(' '));// first, find a space
			if (nLast != -1) {
				CString	str	= strHtmlText.Mid(nFirst, nLast	- nFirst);
				arrUrl.Add(str);
			}
			else {
				nLast =	__MtlFindChar(strHtmlText, nFirst, _T('>'));// second, find	a '>'
				if (nLast != -1) {
					CString	str	= strHtmlText.Mid(nFirst, nLast	- nFirst);
					arrUrl.Add(str);
				}
				else {							// illegal
					break;
				}
			}
		}

		nFirst = nLast;
	}

}
}
